# -*- mode: python -*-
# -*- coding: utf-8 -*-

##############################################################################
#
# ScoDoc
#
# Copyright (c) 1999 - 2023 Emmanuel Viennet.  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Emmanuel Viennet      emmanuel.viennet@viennet.net
#
##############################################################################

"""
Formulaire configuration Module Assiduités
"""
import datetime

from flask_wtf import FlaskForm
from wtforms import DecimalField, SubmitField, ValidationError
from wtforms.fields.simple import StringField
from wtforms.validators import Optional

from wtforms.widgets import TimeInput


class TimeField(StringField):
    """HTML5 time input.
    tiré de : https://gist.github.com/tachyondecay/6016d32f65a996d0d94f
    """

    widget = TimeInput()

    def __init__(self, label=None, validators=None, fmt="%H:%M:%S", **kwargs):
        super(TimeField, self).__init__(label, validators, **kwargs)
        self.fmt = fmt
        self.data = None

    def _value(self):
        if self.raw_data:
            return " ".join(self.raw_data)
        if self.data and isinstance(self.data, str):
            self.data = datetime.time(*map(int, self.data.split(":")))
        return self.data and self.data.strftime(self.fmt) or ""

    def process_formdata(self, valuelist):
        if valuelist:
            time_str = " ".join(valuelist)
            try:
                components = time_str.split(":")
                hour = 0
                minutes = 0
                seconds = 0
                if len(components) in range(2, 4):
                    hour = int(components[0])
                    minutes = int(components[1])

                    if len(components) == 3:
                        seconds = int(components[2])
                else:
                    raise ValueError
                self.data = datetime.time(hour, minutes, seconds)
            except ValueError as exc:
                self.data = None
                raise ValueError(self.gettext("Not a valid time string")) from exc


def check_tick_time(form, field):
    """Le tick_time doit être entre 0 et 60 minutes"""
    if field.data < 1 or field.data > 59:
        raise ValidationError("Valeur de granularité invalide (entre 1 et 59)")


def check_ics_path(form, field):
    """Vérifie que le chemin est bien un chemin absolu
    et qu'il contient edt_id
    """
    data = field.data.strip()
    if not data:
        return
    if not data.startswith("/"):
        raise ValidationError("Le chemin vers les ics doit commencer par /")
    if not "{edt_id}" in data:
        raise ValidationError("Le chemin vers les ics doit utiliser {edt_id}")


class ConfigAssiduitesForm(FlaskForm):
    "Formulaire paramétrage Module Assiduité"

    morning_time = TimeField("Début de la journée")
    lunch_time = TimeField("Heure de midi (date pivot entre Matin et Après Midi)")
    afternoon_time = TimeField("Fin de la journée")

    tick_time = DecimalField(
        "Granularité de la timeline (temps en minutes)",
        places=0,
        validators=[check_tick_time],
    )

    edt_ics_path = StringField(
        label="Chemin vers les ics",
        description="""Chemin absolu unix sur le serveur vers le fichier ics donnant l'emploi
        du temps d'un semestre. La balise <tt>{edt_id}</tt> sera remplacée par l'edt_id du
        semestre (par défaut, son code étape Apogée).
        Si ce champ n'est pas renseigné, les emplois du temps ne seront pas utilisés.""",
        validators=[Optional(), check_ics_path],
    )

    submit = SubmitField("Valider")
    cancel = SubmitField("Annuler", render_kw={"formnovalidate": True})
